@@ -42,9 +42,9 @@ create.ddfobj <- function(model, xmat, meta.data, initial){
     stop("Logistic detection function has been temporarily disabled")
   }
 
-  if(!ddfobj$type %in% c("gamma", "hn", "hr", "unif", "th1", "th2")){
-    stop("Invalid value for detection key function =",ddfobj$type,
-         "  Only hn, hr, gamma, unif, th1, th2 allowed")
+  if(!ddfobj$type %in% c("gamma", "hn", "hr", "unif", "th1", "th2", "tpn")){
+    stop("Invalid value for detection key function =", ddfobj$type,
+         "  Only hn, hr, gamma, unif, th1, th2, tpn allowed")
   }
 
   # Set adjustment function options
@@ -70,14 +70,27 @@ create.ddfobj <- function(model, xmat, meta.data, initial){
       ddfobj$scale <- list(formula="~1")
     }else{
       ddfobj$scale <- list(formula=paste(as.character(modelvalues$formula),
-                                      collapse=""))
+                                         collapse=""))
+      # if we have two-part normal, create the dummy variable for the side
+      # of the apex
+      if(ddfobj$type == "tpn"){
+        if(!grepl("\\.dummy_apex_side", ddfobj$scale[[1]])){
+          ddfobj$scale[[1]] <- paste0(ddfobj$scale[[1]], "+ .dummy_apex_side")
+        }
+        hh <- hist(xmat$distance, plot=FALSE)
+        #xmat$.dummy_apex_side <- factor(xmat$distance <
+        #                                  hh$mids[which.max(hh$count)],
+        #                                levels=c(FALSE, TRUE))
+        xmat$.dummy_apex_side <- as.numeric(!(xmat$distance <
+                                          hh$mids[which.max(hh$count)]))
+      }
     }
   }
 
   if(!is.null(modelvalues$shape.formula)){
     ddfobj$shape <- list(formula=paste(as.character(modelvalues$shape.formula),
                                        collapse=""))
-  }else if(ddfobj$type %in% c("hr", "gamma", "th1", "th2")){
+  }else if(ddfobj$type %in% c("hr", "gamma", "th1", "th2", "tpn")){
     ddfobj$shape <- list(formula=~1)
   }else{
     ddfobj$shape <- NULL
